@@ -22,7 +22,6 @@
 
 #include <config.h>
 
-#include "View.hh"
 #include "BoxMLInkElement.hh"
 #include "BoxMLAttributeSignatures.hh"
 #include "FormattingContext.hh"
@@ -59,6 +58,7 @@ BoxMLInkElement::format(FormattingContext& ctxt)
       res = ctxt.BGD()->getFactory()->ink(res);
       if (oldColor != newColor) res = ctxt.BGD()->getFactory()->color(res, newColor);
       res = ctxt.BGD()->wrapper(ctxt, res);
+      setMaxArea(res);
       setArea(res);
 
       ctxt.pop();
