@@ -22,7 +22,6 @@
 
 #include <config.h>
 
-#include "View.hh"
 #include "BoxMLSpaceElement.hh"
 #include "BoxMLAttributeSignatures.hh"
 #include "FormattingContext.hh"
@@ -64,6 +63,7 @@ BoxMLSpaceElement::format(FormattingContext& ctxt)
 
       AreaRef res = makeSpaceArea(ctxt);
       res = ctxt.BGD()->wrapper(ctxt, res);
+      setMaxArea(res);
       setArea(res);
 
       ctxt.pop();
