@@ -74,12 +74,17 @@ MathMLTokenElement::formatAux(FormattingContext& ctxt)
       else
 	ctxt.setSize(ctxt.MGD()->evaluate(ctxt, ToLength(value), ctxt.getSize()));
     } 
-  
+
+  const unsigned logicalContentLength = GetLogicalContentLength();
   if (SmartPtr<Value> value = GET_ATTRIBUTE_VALUE(MathML, Token, mathvariant))
     ctxt.setVariant(toMathVariant(value));
-  else if (is_a<MathMLIdentifierElement>(SmartPtr<MathMLTokenElement>(this)) && GetLogicalContentLength() == 1)
+  else if (is_a<MathMLIdentifierElement>(SmartPtr<MathMLTokenElement>(this)) && logicalContentLength == 1)
     ctxt.setVariant(ITALIC_VARIANT);
 
+  if (is_a<MathMLTextElement>(SmartPtr<MathMLTokenElement>(this)) ||
+      (is_a<MathMLIdentifierElement>(SmartPtr<MathMLTokenElement>(this)) && logicalContentLength > 1))
+    ctxt.setMathMode(false);
+
   if (SmartPtr<Value> value = GET_ATTRIBUTE_VALUE(MathML, Token, mathcolor))
     ctxt.setColor(ToRGB(value));
 
@@ -109,7 +114,7 @@ MathMLTokenElement::formatAux(FormattingContext& ctxt)
   if (oldColor != newColor)
     res = ctxt.MGD()->getFactory()->color(res, newColor);
 
-  if (!newBackground.transparent && newBackground != oldBackground)
+  if (!newBackground.transparent() && newBackground != oldBackground)
     res = ctxt.MGD()->getFactory()->background(res, newBackground);
 
   return res;
