@@ -64,9 +64,9 @@ typedef gmetadom_MathView MathView;
 #include "MathMLElement.hh"
 #include "MathMLOperatorDictionary.hh"
 #include "MathMLNamespaceContext.hh"
-#if ENABLE_BOXML
+#if GMV_ENABLE_BOXML
 #include "BoxMLNamespaceContext.hh"
-#endif // ENABLE_BOXML
+#endif // GMV_ENABLE_BOXML
 #include "MathGraphicDevice.hh"
 #include "BoxGraphicDevice.hh"
 #include "Gtk_Backend.hh"
@@ -86,10 +86,6 @@ enum SelectState
 enum GtkMathViewProperties
   {
     PROP_DEFAULT_FONT_SIZE,
-    PROP_DEFAULT_FOREGROUND_COLOR,
-    PROP_DEFAULT_BACKGROUND_COLOR,
-    PROP_DEFAULT_SELECTED_FOREGROUND_COLOR,
-    PROP_DEFAULT_SELECTED_BACKGROUND_COLOR,
     PROP_LOG_VERBOSITY
   };
 
@@ -113,13 +109,9 @@ struct _GtkMathViewClass
   GtkMathViewDecorateSignal decorate_over;
 
   AbstractLogger* logger;
-  RGBColor defaultForeground;
-  RGBColor defaultBackground;
-  RGBColor defaultSelectForeground;
-  RGBColor defaultSelectBackground;
+  gint defaultFontSize;
   bool defaultT1OpaqueMode;
   bool defaultT1AntiAliasedMode;
-  Configuration* configuration;
   MathMLOperatorDictionary* dictionary;
   Gtk_Backend* backend;
 };
@@ -181,6 +173,8 @@ static gboolean gtk_math_view_expose_event(GtkWidget*, GdkEventExpose*);
 static gboolean gtk_math_view_button_press_event(GtkWidget*, GdkEventButton*);
 static gboolean gtk_math_view_button_release_event(GtkWidget*, GdkEventButton*);
 static gboolean gtk_math_view_motion_notify_event(GtkWidget*, GdkEventMotion*);
+static gboolean gtk_math_view_focus_in_event(GtkWidget*, GdkEventFocus*);
+static gboolean gtk_math_view_focus_out_event(GtkWidget*, GdkEventFocus*);
 static void     gtk_math_view_realize(GtkWidget*);
 static void     gtk_math_view_size_request(GtkWidget*, GtkRequisition*);
 static void     gtk_math_view_size_allocate(GtkWidget*, GtkAllocation*);
@@ -213,6 +207,10 @@ static guint decorate_over_signal = 0;
 
 /* auxiliary C++ functions */
 
+// static RGBColor
+// RGBColorOfGdkColor(const GdkColor& c)
+// { return RGBColor(c.red >> 8, c.green >> 8, c.blue >> 8); }
+
 static SmartPtr<const Gtk_WrapperArea>
 findGtkWrapperArea(GtkMathView* math_view, GtkMathViewModelId node)
 {
@@ -268,10 +266,6 @@ gtk_math_view_set_property (GObject* object,
   switch (prop_id)
     {
     case PROP_DEFAULT_FONT_SIZE:
-    case PROP_DEFAULT_FOREGROUND_COLOR:
-    case PROP_DEFAULT_BACKGROUND_COLOR:
-    case PROP_DEFAULT_SELECTED_FOREGROUND_COLOR:
-    case PROP_DEFAULT_SELECTED_BACKGROUND_COLOR:
     case PROP_LOG_VERBOSITY:
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -337,11 +331,19 @@ gtk_math_view_paint(GtkMathView* math_view)
     }
 
   rc->setStyle(Gtk_RenderingContext::SELECTED_STYLE);
-  rc->setForegroundColor(math_view_class->defaultSelectForeground);
-  rc->setBackgroundColor(math_view_class->defaultSelectBackground);
+  if (GTK_WIDGET_HAS_FOCUS(math_view))
+    {
+      rc->setForegroundColor(widget->style->text[GTK_STATE_SELECTED]);
+      rc->setBackgroundColor(widget->style->base[GTK_STATE_SELECTED]);
+    }
+  else
+    {
+      rc->setForegroundColor(widget->style->text[GTK_STATE_ACTIVE]);
+      rc->setBackgroundColor(widget->style->base[GTK_STATE_ACTIVE]);
+    }
   rc->setStyle(Gtk_RenderingContext::NORMAL_STYLE);
-  rc->setForegroundColor(math_view_class->defaultForeground);
-  rc->setBackgroundColor(math_view_class->defaultBackground);
+  rc->setForegroundColor(widget->style->fg[GTK_STATE_NORMAL]);
+  rc->setBackgroundColor(widget->style->bg[GTK_STATE_NORMAL]);
 
   gdk_draw_rectangle(math_view->pixmap, widget->style->white_gc, TRUE, 0, 0, width, height);
 
@@ -428,12 +430,8 @@ gtk_math_view_base_class_init(GtkMathViewClass* math_view_class)
 
   SmartPtr<Configuration> configuration = initConfiguration<MathView>(logger, getenv("GTKMATHVIEWCONF"));
   configuration->ref();
-  math_view_class->configuration = configuration;
 
-  math_view_class->defaultForeground = configuration->getRGBColor(logger, "default/color/foreground", DEFAULT_FOREGROUND);
-  math_view_class->defaultBackground = configuration->getRGBColor(logger, "default/color/background", DEFAULT_BACKGROUND);
-  math_view_class->defaultSelectForeground = configuration->getRGBColor(logger, "default/select-color/foreground", DEFAULT_SELECT_FOREGROUND);
-  math_view_class->defaultSelectBackground = configuration->getRGBColor(logger, "default/select-color/background", DEFAULT_SELECT_BACKGROUND);
+  math_view_class->defaultFontSize = configuration->getInt(logger, "default/font-size", DEFAULT_FONT_SIZE);
   math_view_class->defaultT1OpaqueMode = configuration->getBool(logger, "default/t1lib/opaque-mode", false);
   math_view_class->defaultT1AntiAliasedMode = configuration->getBool(logger, "default/t1lib/anti-aliasing", false);
 
@@ -463,12 +461,6 @@ gtk_math_view_base_class_finalize(GtkMathViewClass* math_view_class)
       math_view_class->dictionary = 0;
     }
 
-  if (math_view_class->configuration)
-    {
-      math_view_class->configuration->unref();
-      math_view_class->configuration = 0;
-    }
-
   if (math_view_class->logger)
     {
       math_view_class->logger->unref();
@@ -511,6 +503,8 @@ gtk_math_view_class_init(GtkMathViewClass* math_view_class)
   widget_class->button_press_event = gtk_math_view_button_press_event;
   widget_class->button_release_event = gtk_math_view_button_release_event;
   widget_class->motion_notify_event = gtk_math_view_motion_notify_event;
+  widget_class->focus_in_event = gtk_math_view_focus_in_event;
+  widget_class->focus_out_event = gtk_math_view_focus_out_event;
 
   widget_class->set_scroll_adjustments_signal = 
     g_signal_new("set_scroll_adjustments",
@@ -604,38 +598,6 @@ gtk_math_view_class_init(GtkMathViewClass* math_view_class)
 						   -1,
 						   G_PARAM_READWRITE));
 
-  g_object_class_install_property(gobject_class,
-				  PROP_DEFAULT_FOREGROUND_COLOR,
-				  g_param_spec_string("foreground_color",
-						      "Foreground color",
-						      "The default foreground color",
-						      NULL,
-						      G_PARAM_READWRITE));
-
-  g_object_class_install_property(gobject_class,
-				  PROP_DEFAULT_BACKGROUND_COLOR,
-				  g_param_spec_string("background_color",
-						      "Background color",
-						      "The default background color",
-						      NULL,
-						      G_PARAM_READWRITE));
-
-  g_object_class_install_property(gobject_class,
-				  PROP_DEFAULT_SELECT_FOREGROUND_COLOR,
-				  g_param_spec_string("select_foreground_color",
-						      "Select foreground color",
-						      "The default foreground color for selected markup",
-						      NULL,
-						      G_PARAM_READWRITE));
-
-  g_object_class_install_property(gobject_class,
-				  PROP_DEFAULT_SELECT_BACKGROUND_COLOR,
-				  g_param_spec_string("select_background_color",
-						      "Select background color",
-						      "The default background color for selected markup",
-						      NULL,
-						      G_PARAM_READWRITE));
-
   g_object_class_install_property(gobject_class,
 				  PROP_LOG_VERBOSITY,
 				  g_param_spec_int("log_verbosity",
@@ -693,26 +655,24 @@ gtk_math_view_init(GtkMathView* math_view)
   math_view->top_x = math_view->top_y = 0;
   math_view->old_top_x = math_view->old_top_y = 0;
 
-#if 1
-  SmartPtr<MathView> view = MathView::create();
+  GtkMathViewClass* math_view_class = GTK_MATH_VIEW_CLASS(gtk_type_class(gtk_math_view_get_type()));
+  g_assert(math_view_class != NULL);
+
+  SmartPtr<MathView> view = MathView::create(math_view_class->logger);
   view->ref();
   math_view->view = view;
 
-  GtkMathViewClass* math_view_class = GTK_MATH_VIEW_CLASS(G_OBJECT_GET_CLASS(G_OBJECT(math_view)));
-  g_assert(math_view_class != NULL);
-
-  view->setLogger(math_view_class->logger);
+  view->setDefaultFontSize(math_view_class->defaultFontSize);
   view->setOperatorDictionary(math_view_class->dictionary);
   view->setMathMLNamespaceContext(MathMLNamespaceContext::create(view, math_view_class->backend->getMathGraphicDevice()));
-#if ENABLE_BOXML
+#if GMV_ENABLE_BOXML
   view->setBoxMLNamespaceContext(BoxMLNamespaceContext::create(view, math_view_class->backend->getBoxGraphicDevice()));
-#endif // ENABLE_BOXML
+#endif // GMV_ENABLE_BOXML
 
   math_view->renderingContext = new Gtk_RenderingContext(math_view_class->logger);
   math_view->renderingContext->setColorMap(gtk_widget_get_colormap(GTK_WIDGET(math_view)));
   math_view->renderingContext->setT1OpaqueMode(math_view_class->defaultT1OpaqueMode);
   math_view->renderingContext->setT1AntiAliasedMode(math_view_class->defaultT1AntiAliasedMode);
-#endif
 }
 
 extern "C" GtkWidget*
@@ -813,7 +773,7 @@ gtk_math_view_realize(GtkWidget* widget)
   g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MATH_VIEW (widget));
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED | GTK_CAN_FOCUS);
   math_view = GTK_MATH_VIEW (widget);
 
   attributes.x = widget->allocation.x;
@@ -879,10 +839,24 @@ gtk_math_view_size_allocate(GtkWidget* widget, GtkAllocation* allocation)
 			     allocation->x, allocation->y,
 			     allocation->width, allocation->height);
 
-      gtk_math_view_paint(math_view);
+      //gtk_math_view_paint(math_view);
     }
 }
 
+static gboolean
+gtk_math_view_focus_in_event(GtkWidget* widget, GdkEventFocus*)
+{
+  gtk_math_view_paint(GTK_MATH_VIEW(widget));
+  return FALSE;
+}
+
+static gboolean
+gtk_math_view_focus_out_event(GtkWidget* widget, GdkEventFocus*)
+{
+  gtk_math_view_paint(GTK_MATH_VIEW(widget));
+  return FALSE;
+}
+
 static gint
 gtk_math_view_button_press_event(GtkWidget* widget, GdkEventButton* event)
 {
@@ -1791,3 +1765,10 @@ GTKMATHVIEW_METHOD_NAME(get_t1_anti_aliased_mode)(GtkMathView* math_view)
   g_return_val_if_fail(math_view->renderingContext != 0, FALSE);
   return math_view->renderingContext->getT1AntiAliasedMode() ? TRUE : FALSE;
 }
+
+extern "C" void
+GTKMATHVIEW_METHOD_NAME(add_configuration_path)(const gchar* path)
+{
+  g_return_if_fail(path != NULL);
+  Configuration::addConfigurationPath(path);
+}
