@@ -32,7 +32,7 @@ extern "C" {
 #endif
 
   /* initGUI: some initialization stuff, creates the main window, sets it with a title */
-  void GUI_init(int *, char ***, char *, guint, guint, gint);
+  void GUI_init(int*, const char***, char*, guint, guint, gint);
   void GUI_uninit(void);
 
   int  GUI_load_document(const char*);
